#.------------------------------------------------------------,
#| Compact Flash Builder - IBM Embedded Linux Network Station |
#|                                                            |
#| This code originally developed by:                         |
#| Ryan Harper and Christopher Glendenning                    |
#| March, April 2000                                          |
#|                                                            |
#| Filename: genutils.tcl                                     |
#| Last Modified: May 25, 2000 by Ryan Harper                 |
#| Changes: added waitvar for timing 05.25.00 RAH             |
#|          added cf_partition, cf_format, and cf_mount       |
#|          04.26.00 RAH                                      |  
#|          fixed install overlap 04.26.00 RAH                |
#|          added return vals for procs 04.21.00 RAH          |
#|          modified run_build_script 04.11.00 RAH            |
#|          modified SaveDirInfo 04.10.00 RAH                 |
#| Description: Contains all of the generic functions the     |
#|     Compact Flash Builder uses. Implemented in v2.3 to     |
#|     increase speed and good code design through modularity |
#`------------------------------------------------------------'

#.------------------------------------------------------------,
#| Generic Display Functions                                  |
#|                                                            |
#| Makes a generic text display dialog, uses dialog_create,   |
#| dialog_info                                                |
#`------------------------------------------------------------'
proc textdisplay_create {{title "File 1"}} {
    set top [dialog_create Textdisplay]
    wm title $top $title
    set info [dialog_info $top]
    scrollbar $info.sbar -command "$info.text yview"
    pack $info.sbar -side right -fill y
    text $info.text -wrap word -yscrollcommand "$info.sbar set"
    pack $info.text -side left -expand yes -fill both

    set cntls [dialog_controls $top]
    button $cntls.dismiss -text "Done" -command "exit $top"
    pack $cntls.dismiss -pady 4
    focus $cntls.dismiss

    $info.text configure -state disabled
    $info.text tag configure normal -spacing1 6p -font\
      -*-helvetica-medium-r-normal--*120-*
    $info.text tag configure heading -spacing1 0.2i -font\
      -*-helvetica-bold-r-normal--*-120-*
    $info.text tag configure bold -font -*-helvetica-bold-r-normal--*-120-*
    $info.text tag configure italic -font -*-helvetica-medium-0-normal--*-120-*
    $info.text tag configure typewriter -wrap none -font\
      -*-courier-medium-r-normal--*-120-*
    return $top
}


#.------------------------------------------------------------,
#| Appends text to a generic dialogbox, uses                  |
#| dialog_info						      |
#`------------------------------------------------------------'
proc textdisplay_append {top mesg {tag "normal"}} {
    set info [dialog_info $top]

    $info.text configure -state normal
    $info.text insert end $mesg $tag
    $info.text configure -state disabled
}

#.------------------------------------------------------------,
#| Clears all text from a generic dialogbox, uses             |
#| dialog_info                                                |
#`------------------------------------------------------------'
proc textdisplay_clear {top} {
    set info [dialog_info $top]

    $info.text configure -state normal
    $info.text delete 1.0 end
    $info.text configure -state disabled
}

#.------------------------------------------------------------,
#| Displays text from a file in a generic dialogbox, uses     |
#| dialog_info                                                |                                                           |
#`------------------------------------------------------------'
proc textdisplay_file {top fname} {
    set info [dialog_info $top]
    set fid [open $fname r]
    set contents [read $fid]
    close $fid
    $info.text configure -state normal
    $info.text delete 1.0 end
    $info.text insert end $contents "typewriter"
    $info.text configure -state disabled
}

#.------------------------------------------------------------,
#| Creates a generic dialog box, text area and controls       |
#| section, uses toplevel, wm, frame, pack                    |
#`------------------------------------------------------------'
proc dialog_create {class {win "auto"}} {
    if {$win == "auto"} {
	set count 0
	set win ".dialog[incr count]"
	while {[winfo exists $win]} {
	    set win ".dialog[incr count]"
	}
    }

    toplevel $win -class $class
    #wm geometry $win +250+250
    centerwin $win
    frame $win.info
    pack $win.info -expand yes -fill both -padx 2 -pady 2
    frame $win.sep -height 2 -borderwidth 1 -relief sunken
    pack $win.sep -fill x -pady 4
    frame $win.controls
    pack $win.controls -fill x -padx 4 -pady 4
    wm title $win $class

    wm group $win .
    after idle [format {
		    update idletasks
		    wm minsize %s [winfo reqwidth %s] [winfo reqheight %s]
	    } $win $win $win]
    return $win
}

proc dialog_info {win} {
    return "$win.info"
}

proc dialog_controls {win} {
    return "$win.controls"
}

#.------------------------------------------------------------,
#| makes an OK Cancel dialog box for a function, uses         |
#| dialog_create, dialog_info 				      |
#`------------------------------------------------------------'
proc confirm_ask {mesg {ok "OK"} {cancel "Cancel"}} {

    global confirmStatus
    set top [dialog_create Confirm]
    set info [dialog_info $top]
    #wm geometry $top +250+250
    centerwin $top
    label $info.icon
    pack $info.icon -side left -padx 8 -pady 8
    label $info.mesg -text $mesg
    pack $info.mesg -side right -expand yes -fill both -padx 8 -pady 8
    set cntls [dialog_controls $top]
    button $cntls.ok -text $ok -command {set confirmStatus 1}
    pack $cntls.ok -side left -expand yes
    button $cntls.cancel -text $cancel -command {set confirmStatus 0}
    pack $cntls.cancel -side left -expand yes
    focus $cntls.ok

    wm protocol $top WM_DELETE_WINDOW "$cntls.cancel invoke"
    dialog_wait $top confirmStatus
    destroy $top
    return $confirmStatus
}

#.------------------------------------------------------------,
#| used to wait for computer to respond                       |
#`------------------------------------------------------------'
proc dialog_wait {win varName} {
    dialog_safeguard $win
    set x [expr [winfo rootx .]+50]
    set y [expr [winfo rooty .]+50]
    #    wm geometry $win "+$x+$y"
    centerwin $win
    wm deiconify $win
    grab set $win
    vwait $varName
    grab release $win
    wm withdraw $win
}

proc dialog_safeguard {win} {
    if {[lsearch [bindtags $win] modalDialog] < 0} {
	bindtags $win [linsert [bindtags $win] 0 modalDialog]
    }
}

bind modalDialog <ButtonPress> {
    wm deiconify %W
    raise %W
}

#.------------------------------------------------------------,
#| generic window to show install progress                    |
#| pass it the starting message, the action                   |
#| like "/myscript.sh" and it will exec it,                   |
#| store the output in log, and update window                 |
#| with end mesg                                              |
#`------------------------------------------------------------'
proc action_window {start_mesg action log end_mesg} {
    global lh
    global logfilename
    puts $lh [format "%s" "Starting action_window"]
    puts $lh [format "%s:\n\t--%s\n\t--%s\n\t--%s\n\t--%s" "Passed Vars"\
      "$start_mesg" "$action" "$log" "$end_mesg"]
    flush $lh

    global CFBUILD_DIR_NO_SLASH
    global SCRIPT_DIR
    update
    toplevel .action -width 250 -height 150
    wm resizable .action 0 0
    wm title .action "Executing $action"
    label .action.message -text $start_mesg
    pack .action.message -padx 50 -pady 50
    #wm geometry .action +360+340
    centerwin .action
    update
    cd $CFBUILD_DIR_NO_SLASH/$SCRIPT_DIR
    puts $lh [format "%s" "Executing $action"]
    flush $lh
    exec $action >>&$CFBUILD_DIR_NO_SLASH/Log/$logfilename
    set lh [open $CFBUILD_DIR_NO_SLASH/Log/$logfilename a]
    cd $CFBUILD_DIR_NO_SLASH
    .action.message configure -text $end_mesg
    after 1000
    destroy .action
    return
}

#.------------------------------------------------------------,
#| same as action_window, but execs a proc, instead           |
#| of a script                                                |
#`------------------------------------------------------------'
proc action_window_proc {start_mesg action log end_mesg} {
    global lh
    puts $lh [format "%s" "Starting action_window_proc"]
    puts $lh [format "%s:\n\t--%s\n\t--%s\n\t--%s\n\t--%s" "Passed Vars"\
      "$start_mesg" "$action" "$log" "$end_mesg"]
    flush $lh

    global CFBUILD_DIR_NO_SLASH
    global SCRIPT_DIR
    update
    toplevel .action_proc -width 250 -height 150
    wm resizable .action_proc 0 0
    wm title .action_proc " "
    label .action_proc.message -text $start_mesg
    pack .action_proc.message -padx 50 -pady 50
    #wm geometry .action_proc +360+340
    centerwin .action_proc
    update
    cd $CFBUILD_DIR_NO_SLASH/$SCRIPT_DIR
    ${action}
    cd $CFBUILD_DIR_NO_SLASH
    .action_proc.message configure -text $end_mesg
    after 1000
    destroy .action_proc
    return
}

#.------------------------------------------------------------,
#| center the windows 					      |
#`------------------------------------------------------------'
proc centerwin {winname} {
    set xmax [winfo screenwidth $winname]
    set ymax [winfo screenheight $winname]
    set x [expr ($xmax-[winfo reqwidth $winname])/2]
    set y [expr ($ymax-[winfo reqheight $winname])/2]
    wm geometry $winname "+$x+$y"
}

#.------------------------------------------------------------,
#| check to see if window exits				      |
#`------------------------------------------------------------'
proc check_for_win {winname} {
    if {[winfo exists $winname]==1} {
	set temp [focus -lastfor $winname]
	focus $temp
	raise $winname
	return true
    } {
	return false
    }
}

#.------------------------------------------------------------,
#| cf_partition will dynamically determine how to partition   |
#| the flash that is inserted.  It runs an sfdisk -g to get   |
#| the geometry from the flash,and then calculates what the   |
#| sizes should be for each flash.  This is important as      |
#| all Flashes are not created equal. Each one claims to      |
#| be a 32MB flash, but, they might have 4 heads and XX       |
#| cylinders, where another will have 8 heads and YY cylinders|
#| What this means is that each one is different, so you have |
#| partition them different.                                  |
#`------------------------------------------------------------'
proc cf_partition {} {
    global CFBUILD_DIR_NO_SLASH
    global SCRIPT_DIR
    global lh
    global logfilename

    puts $lh [format "%s" "Starting cf_partition"]
    flush $lh

    #--- try to unmount flash
    puts $lh [format "%s" "\tUnmounting devices"]
    flush $lh
    update
    exec $CFBUILD_DIR_NO_SLASH/$SCRIPT_DIR/cf_umount.sh &
    after 1000
    set lh [open $CFBUILD_DIR_NO_SLASH/Log/$logfilename a]

    #--- get geometry from flash
    cd $CFBUILD_DIR_NO_SLASH/Log
    set filename "sfdisk.out"
    set fh [open $filename w]
    fconfigure $fh

     exec sfdisk -g /dev/hda >@ $fh 
     close $fh

    #--- read geometry and set vars
    set sfh [open $filename r]
    set string [gets $sfh]
    close $sfh
    set list [split $string]
    set cyl [lindex $list 1]
    set head [lindex $list 3]
    set sect [lindex $list 5]
    set ram_disk_size "32"
    set size_var [expr $cyl * $head]
    set total_size [expr $cyl * $head * $sect]
    set part1_start $sect
    set part2_start [expr {[expr $cyl - $ram_disk_size]} * $head * $sect]
    set part1_size [expr $part2_start - 32]
    set part2_size [expr $total_size - $part2_start]

    #--- write calculation out to file to be used with sfdisk to set partition
    set pfname "partition.out"
    set pfh [open $pfname w]
    fconfigure $pfh
    puts $pfh [format "%s" "# partition table of /dev/hda "]
    puts $pfh [format "%s" "unit: sectors"]
    puts $pfh [format "%s" "\n"]
    puts $pfh [format "%s" "/dev/hda1 : start= $part1_start, size=\
      $part1_size, Id=83"]
    puts $pfh [format "%s" "/dev/hda2 : start= $part2_start, size=\
      $part2_size, Id=83"]
    puts $pfh [format "%s" "/dev/hda3 : start=        0, size=       0, Id= 0"]
    puts $pfh [format "%s" "/dev/hda4 : start=        0, size=       0, Id= 0"]
    flush $pfh
    close $pfh

    #--- create window to show progress
    update
    toplevel .top -width 250 -height 150
    wm title .top "Partitioning Flash"
    label .top.message -text "Partitioning... Please Wait."
    pack .top.message -padx 50 -pady 50
    wm geometry .top +360+340
    update idletasks

    exec sfdisk --force /dev/hda < partition.out >>&$logfilename 

    .top.message configure -text "Complete"
    after 1000
    destroy .top
    update

    set lh [open $CFBUILD_DIR_NO_SLASH/Log/$logfilename a]
    puts $lh [format "%s" "auto_partition complete"]
    flush $lh
    return
}

#.------------------------------------------------------------,
#| cf_format will format the compact flash with the ext2 fs.  |
#`------------------------------------------------------------'
proc cf_format {} {
    global CFBUILD_DIR_NO_SLASH
    global SCRIPT_DIR
    global lh
    global logfilename

    puts $lh [format "%s" "Starting cf_format"]
    flush $lh

    #--- try to unmount the flash
    puts $lh [format "%s" "\tUnmounting devices"]
    flush $lh
    update
    exec $CFBUILD_DIR_NO_SLASH/$SCRIPT_DIR/cf_umount.sh &
    after 1000
    set lh [open $CFBUILD_DIR_NO_SLASH/Log/$logfilename a]
    puts $lh [format "%s" "\tFormating"]
    flush $lh

    #--- create window to show progress
    update
    toplevel .top -width 250 -height 150
    wm title .top "Formatting Flash"
    label .top.message -text "Formatting Flash... Please Wait."
    pack .top.message -padx 50 -pady 50
    wm geometry .top +360+340
    update idletasks

    catch { exec mke2fs /dev/hda1 >> $logfilename } err
    if {$err != 0} { 
	#tk_messageBox -message "$err \n Make sure you have a Flash Installed"
	destroy .top
	return -1
    }
    catch { exec mke2fs /dev/hda2 >> $logfilename } err
    if {$err != 0} { 
	tk_messageBox -message $err
	destroy .top
	return -1
    }
    .top.message configure -text "Complete"
    after 1000
    destroy .top
    update

    set lh [open $CFBUILD_DIR_NO_SLASH/Log/$logfilename a]
    puts $lh [format "%s" "cf_format complete"]
    flush $lh
    return 1
}

#.------------------------------------------------------------,
#| cf_mount will mount the first 2 partitions of a flash to   |
#| /mnt/hd1, and /mnt/hd2 respectively using /dev/hda1,hda2   |
#`------------------------------------------------------------'
proc cf_mount {} {
    global CFBUILD_DIR_NO_SLASH
    global SCRIPT_DIR
    global lh
    global logfilename

    puts $lh [format "%s" "Starting cf_mount"]
    flush $lh

    #--- make window to show progress
    update
    toplevel .top -width 250 -height 150
    wm title .top "Mounting Flash"
    label .top.message -text "Mounting Flash... Please Wait."
    pack .top.message -padx 50 -pady 50
    wm geometry .top +360+340
    update idletasks

    #--- try to mount flash
    puts $lh [format "%s" "\tMounting devices"]
    flush $lh
    update
    exec $CFBUILD_DIR_NO_SLASH/$SCRIPT_DIR/cf_mount.sh &
    set lh [open $CFBUILD_DIR_NO_SLASH/Log/$logfilename a]
    .top.message configure -text "Complete"
    after 1000
    destroy .top
    update
    puts $lh [format "%s" "cf_mount complete"]
    flush $lh
    return
}

#.------------------------------------------------------------,
#| generic install program function, pass message,            |
#`------------------------------------------------------------'
proc install_program {mesg build_cmd help_cmd} {
    global lh
    puts $lh [format "%s" "Starting install_program"]
    puts $lh [format "%s:\n\t--%s\n\t--%s\n\t--%s" "Passed Vars" "$mesg"\
      "$build_cmd" "$help_cmd"]
    flush $lh

    .win.right.startmesg configure -justify "left" -text $mesg
    update
    if {[winfo exists .win.rightbottom.build] == 0} {
	button .win.rightbottom.build -text {Build Package} -state normal\
	  -command $build_cmd
	place .win.rightbottom.build -x 0 -y 10
	return
    } {
	.win.rightbottom.build configure -text {Build Package} -state normal\
	  -command $build_cmd
    }
    .win.rightbottom.install configure -width 15 -text {Help} -state normal\
      -command $help_cmd
}

#.------------------------------------------------------------,
#| makes a new window with a text area in it and reads infile |
#| to populate it with text from infile.  Adds a Close button |
#| at bottom of window.                                       |
#`------------------------------------------------------------'
proc show_help_file {infile winname} {
    global CFBUILD_DIR_NO_SLASH
    global INSTALLDOC_DIR
    global lh
    puts $lh [format "%s" "Starting show_help_file"]
    puts $lh [format "%s:\n\t--%s\n\t--%s" "Passed Vars" "$infile" "$winname"]
    flush $lh


    cd $CFBUILD_DIR_NO_SLASH/$INSTALLDOC_DIR
    set in $infile
    set win [textdisplay_create $winname]
    textdisplay_file $win $in
    cd $CFBUILD_DIR_NO_SLASH
}

#.------------------------------------------------------------,
#| view_log uses the same method as the show_help_file, but   |
#| takes no args, and uses the $logfilename to display the log|
#| of all current operations.                                 |
#`------------------------------------------------------------'
proc view_log {} {
    global CFBUILD_DIR_NO_SLASH
    global lh
    global logfilename
    puts $lh [format "%s" "Starting view_log"]
    flush $lh


    set in $CFBUILD_DIR_NO_SLASH/Log/$logfilename
    set win [textdisplay_create $logfilename]
    textdisplay_file $win $in
}

#.------------------------------------------------------------,
#| get_dir is a generic function used to get file locations   |
#| setup variables for scripts (save_type) and can read in    |
#| multiple files, for example, netscape 4.61 has 2 rpms,     |
#| netscape,and netscape-common, so, to install 4.61, you need|
#| both.  get_dir facilitates this by taking a numeric arg,   |
#| num_browse, this value sets the number of entry boxes to   |
#| read values from. Finally, after reading in these values,  |
#| CFBuilder wants to install or unpack something, so, cmd is |
#| executed after reading each browse box.  The values from   |
#| each entry box are read into a_path() global array. Each   |
#| item read is put in incrementally, so, forexample, netscape|
#| as 2 rpms, when you are entrying both paths, entry box 0   |
#| stores the value in a_path(i), where i is a var ranging    |
#| from 0 < num_browse.                                       |
#`------------------------------------------------------------'
proc get_dir {prompt save_type num_browse cmd} {
    global lh
    puts $lh [format "%s" "Starting get_dir"]
    puts $lh [format "%s:\n\t--%s\n\t--%s\n\t--%s" "Passed Vars" "$prompt"\
      "$save_type" "$num_browse" "$cmd"]
    flush $lh

    #--- set up globals,
    global i			;#--- index into a_path,
    global types		;#--- array with file types, .rpm,.gz,.tar etc... for FileOpenDialog
    global a_path		;#--- array of file paths
    global return_val		;#--- used for modality, probably useless though, need to test removing it
    set return_val 0

    #--- check to see if get_dir is already running... 
    if {[winfo exists .win.]} {
	wm deiconify .win
	return
    }

    #--- setting window height based on number of entry boxes required, 
    #--- works reasonalby well for less than 5.  havent tested more.
    set wh [expr 70 + $num_browse * 38]

    #--- make window
    toplevel .win.top -width 250 -height $wh
    wm title .win.top " "
    label .win.top.message -text $prompt
    centerwin .win.top

    #--- make button with passed value cmd as its command
    button .win.top.done -width 7 -text {Done} -command "${cmd} ${save_type}\
      ${num_browse}"

    #--- setting up vars for spacing of the entry boxes... 
    set e_y_val "50"
    set e_space "28"
    set b_y_val "45"
    set b_space "30"

    #--- forloop to set the browse buttons, so, if you ask for 2 entrys, it sets
    #--- the browse command to take the entry box number as an argument to keep 
    #--- track of where the value from the entry box goes into a_path(i)
    for {set i 0} {$i < $num_browse} {incr i} {
	button .win.top.browse$i -width 7 -text "Browse" -command {browse "$i"}
	entry .win.top.entry$i
	place .win.top.entry$i -x 15 -y $e_y_val
	place .win.top.browse$i -x 170 -y $b_y_val
	set e_y_val [expr $e_y_val + $e_space]
	set b_y_val [expr $b_y_val + $b_space]
    }
   
    #--- reset index to 0
    set i "0"
    place .win.top.message -x 10 -y 12
    place .win.top.done -x 170 -y [expr $wh - 32]


    #--- command launches a FileOpenDialog, saves path in a_path(i)
    proc browse {index} {
	global i
	global types
	global a_path
	set a_path($index) [tk_getOpenFile -title " "\
	  -filetypes $types -parent .win.top]
	.win.top.entry$index insert 0 $a_path($index)
	.win.top.entry$index select range 0 [string length $a_path($index)]
	focus .win.top.entry$index
	incr i
	return

    }

   

}

#.------------------------------------------------------------,
#| destroy win var                                            |
#`------------------------------------------------------------'
proc exit {win} {
    global waitvar
    global lh
    puts $lh [format "%s" "Exiting Program (command=destroy $win)"]
    flush $lh
    destroy $win
    set waitvar 0
}

#.------------------------------------------------------------,
#| confirm destruction of Win, modal dialog                   |
#`------------------------------------------------------------'
proc cexit {win} {
    global waitvar
    global lh
    puts $lh [format "%s" "Exiting Program (command=confirm_exit)"]
    flush $lh

    if {[confirm_ask "Do you really want to quit?"]} {
	set waitvar 0
	destroy $win
    }
}

#.------------------------------------------------------------,
#| required function for world domination		      |
#`------------------------------------------------------------'
proc MOO {} {
    global lh
    puts $lh [format "%s" "Exiting Program (command=MOO - You have been\
      destroyed by a cow)"]
    flush $lh
    destroy .
}

#.------------------------------------------------------------,
#| proc to change wizard pages, takes what page to change to  |
#| the value runwizard which sets whether wizard buttons are  |
#| visible or not.					      |
#`------------------------------------------------------------'
proc ChangePage {nextwin runwizard} {
    global lh
    puts $lh [format "%s-> %s" "Changing Pages" "$nextwin"]
    flush $lh

    destroy .win
    $nextwin $runwizard
}

#.------------------------------------------------------------,
#| possible dead function, used to set up environmental vars  |
#`------------------------------------------------------------'
proc SaveInstallDir {var1 var2} {
    global lh
    puts $lh [format "%s" "Starting SaveInstallDir"]
    puts $lh [format "%s:\n\t--%s\n\t--%s" "Passed Vars" "$var1" "$var2"]
    flush $lh

    set filename "/root/IBM/InstallDir"
    set fd [open $filename w]
    fconfigure $fd
    puts $fd [format "%s=%s" "install_dest" $var1]
    puts $fd [format "%s=%s" "install_source" $var2]
    close $fd
    exec chmod a+x /root/IBM/InstallDir
    exec sync
}

